@@ -22,6 +22,7 @@ gcheckout         GTK+ checkout.
 gcommit           GTK+ commit dialog.
 gconflicts        GTK+ conflicts. 
 gdiff             Show differences in working tree in a GTK+ Window. 
+ghandle-patch     Display and optionally merge a merge directive or patch.
 ginit             Initialise a new branch.
 gmissing          GTK+ missing revisions dialog. 
 gpreferences      GTK+ preferences dialog. 
@@ -36,7 +37,7 @@ import sys
 
 import bzrlib
 
-version_info = (0, 93, 0, 'final', 0)
+version_info = (0, 94, 0, 'final', 1)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]
@@ -44,19 +45,14 @@ else:
     version_string = '%d.%d.%d%s%d' % version_info
 __version__ = version_string
 
+required_bzrlib = (1, 3)
+
 def check_bzrlib_version(desired):
     """Check that bzrlib is compatible.
 
     If version is < bzr-gtk version, assume incompatible.
-    If version == bzr-gtk version, assume completely compatible
-    If version == bzr-gtk version + 1, assume compatible, with deprecations
-    Otherwise, assume incompatible.
     """
-    desired_plus = (desired[0], desired[1]+1)
     bzrlib_version = bzrlib.version_info[:2]
-    if bzrlib_version == desired or (bzrlib_version == desired_plus and
-                                     bzrlib.version_info[3] == 'dev'):
-        return
     try:
         from bzrlib.trace import warning
     except ImportError:
@@ -67,14 +63,10 @@ def check_bzrlib_version(desired):
         warning('Installed Bazaar version %s is too old to be used with bzr-gtk'
                 ' %s.' % (bzrlib.__version__, __version__))
         raise BzrError('Version mismatch: %r, %r' % (version_info, bzrlib.version_info) )
-    else:
-        warning('bzr-gtk is not up to date with installed bzr version %s.'
-                ' \nThere should be a newer version available, e.g. %i.%i.' 
-                % (bzrlib.__version__, bzrlib_version[0], bzrlib_version[1]))
 
 
 if version_info[2] == "final":
-    check_bzrlib_version(version_info[:2])
+    check_bzrlib_version(required_bzrlib)
 
 from bzrlib.trace import warning
 if __name__ != 'bzrlib.plugins.gtk':
@@ -86,6 +78,7 @@ from bzrlib import (
     branch,
     builtins,
     errors,
+    merge_directive,
     workingtree,
     )
 """)
@@ -116,6 +109,17 @@ def data_path():
     return os.path.dirname(__file__)
 
 
+def icon_path(*args):
+    basedirs = [os.path.join(data_path()),
+             "/usr/share/bzr-gtk", 
+             "/usr/local/share/bzr-gtk"]
+    for basedir in basedirs:
+        path = os.path.join(basedir, 'icons', *args)
+        if os.path.exists(path):
+            return path
+    return None
+
+
 class GTKCommand(Command):
     """Abstract class providing GTK specific run commands."""
 
@@ -222,13 +226,13 @@ class cmd_gdiff(GTKCommand):
             wt.unlock()
 
 
-def start_viz_window(branch, revision, limit=None):
+def start_viz_window(branch, revisions, limit=None):
     """Start viz on branch with revision revision.
     
     :return: The viz window object.
     """
-    from viz.branchwin import BranchWindow
-    return BranchWindow(branch, revision, limit)
+    from viz import BranchWindow
+    return BranchWindow(branch, revisions, limit)
 
 
 class cmd_visualise(Command):
@@ -244,21 +248,23 @@ class cmd_visualise(Command):
         "revision",
         Option('limit', "Maximum number of revisions to display.",
                int, 'count')]
-    takes_args = [ "location?" ]
+    takes_args = [ "locations*" ]
     aliases = [ "visualize", "vis", "viz" ]
 
-    def run(self, location=".", revision=None, limit=None):
+    def run(self, locations_list, revision=None, limit=None):
         set_ui_factory()
-        (br, path) = branch.Branch.open_containing(location)
-        if revision is None:
-            revid = br.last_revision()
-            if revid is None:
-                return
-        else:
-            (revno, revid) = revision[0].in_history(br)
-
+        if locations_list is None:
+            locations_list = ["."]
+        revids = []
+        for location in locations_list:
+            (br, path) = branch.Branch.open_containing(location)
+            if revision is None:
+                revids.append(br.last_revision())
+            else:
+                (revno, revid) = revision[0].in_history(br)
+                revids.append(revid)
         import gtk
-        pp = start_viz_window(br, revid, limit)
+        pp = start_viz_window(br, revids, limit)
         pp.connect("destroy", lambda w: gtk.main_quit())
         pp.show()
         gtk.main()
@@ -311,9 +317,8 @@ class cmd_gannotate(GTKCommand):
         else:
             revision_id = getattr(tree, 'get_revision_id', lambda: None)()
 
-        window = GAnnotateWindow(all, plain)
+        window = GAnnotateWindow(all, plain, branch=br)
         window.connect("destroy", lambda w: gtk.main_quit())
-        window.set_title(path + " - gannotate")
         config = GAnnotateConfig(window)
         window.show()
         br.lock_read()
@@ -378,14 +383,24 @@ class cmd_gstatus(GTKCommand):
     
     aliases = [ "gst" ]
     takes_args = ['PATH?']
-    takes_options = []
+    takes_options = ['revision']
 
-    def run(self, path='.'):
+    def run(self, path='.', revision=None):
         import os
         gtk = self.open_display()
         from status import StatusDialog
         (wt, wt_path) = workingtree.WorkingTree.open_containing(path)
-        status = StatusDialog(wt, wt_path)
+        
+        if revision is not None:
+            try:
+                revision_id = revision[0].in_history(wt.branch).rev_id
+            except:
+                from bzrlib.errors import BzrError
+                raise BzrError('Revision %r doesn\'t exist' % revision[0].user_spec )
+        else:
+            revision_id = None
+
+        status = StatusDialog(wt, wt_path, revision_id)
         status.connect("destroy", gtk.main_quit)
         status.run()
 
@@ -522,7 +537,7 @@ class cmd_commit_notify(GTKCommand):
         from notify import NotifyPopupMenu
         gtk = self.open_display()
         menu = NotifyPopupMenu()
-        icon = gtk.status_icon_new_from_file(os.path.join(data_path(), "bzr-icon-64.png"))
+        icon = gtk.status_icon_new_from_file(icon_path("bzr-icon-64.png"))
         icon.connect('popup-menu', menu.display)
 
         import cgi
@@ -535,12 +550,8 @@ class cmd_commit_notify(GTKCommand):
         from bzrlib.transport import get_transport
         if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
             import dbus.glib
-        from bzrlib.plugins.dbus import activity
+        BROADCAST_INTERFACE = "org.bazaarvcs.plugins.dbus.Broadcast"
         bus = dbus.SessionBus()
-        # get the object so we can subscribe to callbacks from it.
-        broadcast_service = bus.get_object(
-            activity.Broadcast.DBUS_NAME,
-            activity.Broadcast.DBUS_PATH)
 
         def catch_branch(revision_id, urls):
             # TODO: show all the urls, or perhaps choose the 'best'.
@@ -577,8 +588,9 @@ class cmd_commit_notify(GTKCommand):
             except Exception, e:
                 print e
                 raise
-        broadcast_service.connect_to_signal("Revision", catch_branch,
-            dbus_interface=activity.Broadcast.DBUS_INTERFACE)
+        bus.add_signal_receiver(catch_branch,
+                                dbus_interface=BROADCAST_INTERFACE,
+                                signal_name="Revision")
         pynotify.init("bzr commit-notify")
         gtk.main()
 
@@ -681,6 +693,43 @@ class cmd_test_gtk(GTKCommand):
 register_command(cmd_test_gtk)
 
 
+class cmd_ghandle_patch(GTKCommand):
+    """Display a patch or merge directive, possibly merging.
+
+    This is a helper, meant to be launched from other programs like browsers
+    or email clients.  Since these programs often do not allow parameters to
+    be provided, a "handle-patch" script is included.
+    """
+
+    takes_args = ['path']
+
+    def run(self, path):
+        try:
+            from bzrlib.plugins.gtk.diff import (DiffWindow,
+                                                 MergeDirectiveWindow)
+            lines = open(path, 'rb').readlines()
+            lines = [l.replace('\r\n', '\n') for l in lines]
+            try:
+                directive = merge_directive.MergeDirective.from_lines(lines)
+            except errors.NotAMergeDirective:
+                window = DiffWindow()
+                window.set_diff_text(path, lines)
+            else:
+                window = MergeDirectiveWindow(directive, path)
+                window.set_diff_text(path, directive.patch.splitlines(True))
+            window.show()
+            gtk = self.open_display()
+            window.connect("destroy", gtk.main_quit)
+        except Exception, e:
+            from dialog import error_dialog
+            error_dialog('Error', str(e))
+            raise
+        gtk.main()
+
+
+register_command(cmd_ghandle_patch)
+
+
 import gettext
 gettext.install('olive-gtk')
 
