@@ -23,15 +23,15 @@ except:
 import bzrlib
 import gtk
 import os
-from bzrlib.plugins.gtk import data_path
+from bzrlib.plugins.gtk import icon_path
 
 class AboutDialog(gtk.AboutDialog):
     def __init__(self):
         super(AboutDialog, self).__init__()
-        self.set_name("Bazaar")
-        self.set_version(bzrlib.version_string)
-        self.set_website("http://bazaar-vcs.org/")
-        self.set_license("GNU GPLv2")
-        self.set_icon(gtk.gdk.pixbuf_new_from_file(os.path.join(data_path(), "bzr-icon-64.png")))
+        self.set_name("Bazaar GTK")
+        self.set_version(bzrlib.plugins.gtk.version_string)
+        self.set_website("http://bazaar-vcs.org/BzrGtk")
+        self.set_license("GNU GPL v2 or later")
+        self.set_icon(gtk.gdk.pixbuf_new_from_file(icon_path("bzr-icon-64.png")))
         self.connect ("response", lambda d, r: d.destroy())
 
