@@ -14,12 +14,13 @@ import gobject
 import pango
 
 from bzrlib.plugins.gtk.window import Window
+from bzrlib.plugins.gtk import icon_path
 from bzrlib.plugins.gtk.tags import AddTagDialog
 from bzrlib.plugins.gtk.preferences import PreferencesWindow
-from bzrlib.revision import Revision
+from bzrlib.plugins.gtk.branchview import TreeView, treemodel
+from bzrlib.revision import Revision, NULL_REVISION
 from bzrlib.config import BranchConfig
 from bzrlib.config import GlobalConfig
-from treeview import TreeView
 
 class BranchWindow(Window):
     """Branch window.
@@ -28,11 +29,11 @@ class BranchWindow(Window):
     for a particular branch.
     """
 
-    def __init__(self, branch, start, maxnum, parent=None):
+    def __init__(self, branch, start_revs, maxnum, parent=None):
         """Create a new BranchWindow.
 
         :param branch: Branch object for branch to show.
-        :param start: Revision id of top revision.
+        :param start_revs: Revision ids of top revisions.
         :param maxnum: Maximum number of revisions to display, 
                        None for no limit.
         """
@@ -41,7 +42,7 @@ class BranchWindow(Window):
         self.set_border_width(0)
 
         self.branch      = branch
-        self.start       = start
+        self.start_revs  = start_revs
         self.maxnum      = maxnum
         self.config      = GlobalConfig()
 
@@ -100,7 +101,6 @@ class BranchWindow(Window):
 
         vbox.pack_start(self.construct_menubar(), expand=False, fill=True)
         vbox.pack_start(self.construct_navigation(), expand=False, fill=True)
-        vbox.pack_start(self.construct_loading_msg(), expand=False, fill=True)
         
         vbox.pack_start(self.paned, expand=True, fill=True)
         vbox.set_focus_child(self.paned)
@@ -156,8 +156,16 @@ class BranchWindow(Window):
         view_menu.add(view_menu_compact)
         view_menu.add(gtk.SeparatorMenuItem())
 
-        for (label, name) in [("Revision _Number", "revno"), ("_Date", "date")]:
-            col = gtk.CheckMenuItem("Show " + label + " Column")
+        self.mnu_show_revno_column = gtk.CheckMenuItem("Show Revision _Number Column")
+        self.mnu_show_date_column = gtk.CheckMenuItem("Show _Date Column")
+
+        # Revision numbers are pointless if there are multiple branches
+        if len(self.start_revs) > 1:
+            self.mnu_show_revno_column.set_sensitive(False)
+            self.treeview.set_property('revno-column-visible', False)
+
+        for (col, name) in [(self.mnu_show_revno_column, "revno"), 
+                            (self.mnu_show_date_column, "date")]:
             col.set_active(self.treeview.get_property(name + "-column-visible"))
             col.connect('toggled', self._col_visibility_changed, name)
             view_menu.add(col)
@@ -170,27 +178,16 @@ class BranchWindow(Window):
         go_menu_next = self.next_rev_action.create_menu_item()
         go_menu_prev = self.prev_rev_action.create_menu_item()
 
-        tags_menu = gtk.Menu()
-        go_menu_tags = gtk.MenuItem("_Tags")
-        go_menu_tags.set_submenu(tags_menu)
-
-        if self.branch.supports_tags():
-            tags = self.branch.tags.get_tag_dict().items()
-            tags.sort()
-            tags.reverse()
-            for tag, revid in tags:
-                tag_item = gtk.MenuItem(tag)
-                tag_item.connect('activate', self._tag_selected_cb, revid)
-                tags_menu.add(tag_item)
-
-            go_menu_tags.set_sensitive(len(tags) != 0)
-        else:
-            go_menu_tags.set_sensitive(False)
+        tag_image = gtk.Image()
+        tag_image.set_from_file(icon_path("tag-16.png"))
+        self.go_menu_tags = gtk.ImageMenuItem("_Tags")
+        self.go_menu_tags.set_image(tag_image)
+        self._update_tags()
 
         go_menu.add(go_menu_next)
         go_menu.add(go_menu_prev)
         go_menu.add(gtk.SeparatorMenuItem())
-        go_menu.add(go_menu_tags)
+        go_menu.add(self.go_menu_tags)
 
         revision_menu = gtk.Menu()
         revision_menuitem = gtk.MenuItem("_Revision")
@@ -198,13 +195,18 @@ class BranchWindow(Window):
 
         revision_menu_diff = gtk.MenuItem("View Changes")
         revision_menu_diff.connect('activate', 
-                lambda w: self.treeview.show_diff())
+                self._menu_diff_cb)
+        
+        revision_menu_compare = gtk.MenuItem("Compare with...")
+        revision_menu_compare.connect('activate',
+                self._compare_with_cb)
 
         revision_menu_tag = gtk.MenuItem("Tag Revision")
         revision_menu_tag.connect('activate', self._tag_revision_cb)
 
         revision_menu.add(revision_menu_tag)
         revision_menu.add(revision_menu_diff)
+        revision_menu.add(revision_menu_compare)
 
         branch_menu = gtk.Menu()
         branch_menuitem = gtk.MenuItem("_Branch")
@@ -213,50 +215,45 @@ class BranchWindow(Window):
         branch_menu.add(gtk.MenuItem("Pu_ll Revisions"))
         branch_menu.add(gtk.MenuItem("Pu_sh Revisions"))
 
+        help_menu = gtk.Menu()
+        help_menuitem = gtk.MenuItem("_Help")
+        help_menuitem.set_submenu(help_menu)
+
+        help_about_menuitem = gtk.ImageMenuItem(gtk.STOCK_ABOUT, self.accel_group)
+        help_about_menuitem.connect('activate', self._about_dialog_cb)
+
+        help_menu.add(help_about_menuitem)
+
         menubar.add(file_menuitem)
         menubar.add(edit_menuitem)
         menubar.add(view_menuitem)
         menubar.add(go_menuitem)
         menubar.add(revision_menuitem)
         menubar.add(branch_menuitem)
+        menubar.add(help_menuitem)
         menubar.show_all()
 
         return menubar
-    
-    def construct_loading_msg(self):
-        image_loading = gtk.image_new_from_stock(gtk.STOCK_REFRESH,
-                                                 gtk.ICON_SIZE_BUTTON)
-        image_loading.show()
-        
-        label_loading = gtk.Label(_("Please wait, loading ancestral graph..."))
-        label_loading.set_alignment(0.0, 0.5)
-        label_loading.show()
-        
-        self.loading_msg_box = gtk.HBox()
-        self.loading_msg_box.set_spacing(5)
-        self.loading_msg_box.set_border_width(5)        
-        self.loading_msg_box.pack_start(image_loading, False, False)
-        self.loading_msg_box.pack_start(label_loading, True, True)
-        self.loading_msg_box.show()
-        
-        return self.loading_msg_box
 
     def construct_top(self):
         """Construct the top-half of the window."""
         # FIXME: Make broken_line_length configurable
 
-        self.treeview = TreeView(self.branch, self.start, self.maxnum, self.compact_view)
+        self.treeview = TreeView(self.branch, self.start_revs, self.maxnum, self.compact_view)
 
         self.treeview.connect('revision-selected',
                 self._treeselection_changed_cb)
+        self.treeview.connect('revision-activated',
+                self._tree_revision_activated)
 
-        self.treeview.connect('revisions-loaded', 
-                lambda x: self.loading_msg_box.hide())
+        self.treeview.connect('tag-added', lambda w, t, r: self._update_tags())
 
         for col in ["revno", "date"]:
             option = self.config.get_user_option(col + '-column-visible')
             if option is not None:
                 self.treeview.set_property(col + '-column-visible', option == 'True')
+            else:
+                self.treeview.set_property(col + '-column-visible', False)
 
         self.treeview.show()
 
@@ -291,12 +288,13 @@ class BranchWindow(Window):
     def construct_bottom(self):
         """Construct the bottom half of the window."""
         from bzrlib.plugins.gtk.revisionview import RevisionView
-        self.revisionview = RevisionView(None, tags=[], show_children=True, branch=self.branch)
+        self.revisionview = RevisionView(branch=self.branch)
         (width, height) = self.get_size()
         self.revisionview.set_size_request(width, int(height / 2.5))
         self.revisionview.show()
         self.revisionview.set_show_callback(self._show_clicked_cb)
-        self.revisionview.set_go_callback(self._go_clicked_cb)
+        self.revisionview.connect('notify::revision', self._go_clicked_cb)
+        self.treeview.connect('tag-added', lambda w, t, r: self.revisionview.update_tags())
         return self.revisionview
 
     def _tag_selected_cb(self, menuitem, revid):
@@ -308,20 +306,21 @@ class BranchWindow(Window):
         parents  = self.treeview.get_parents()
         children = self.treeview.get_children()
 
-        if revision is not None:
+        if revision and revision != NULL_REVISION:
             prev_menu = gtk.Menu()
             if len(parents) > 0:
                 self.prev_rev_action.set_sensitive(True)
                 for parent_id in parents:
-                    parent = self.branch.repository.get_revision(parent_id)
-                    try:
-                        str = ' (' + parent.properties['branch-nick'] + ')'
-                    except KeyError:
-                        str = ""
+                    if parent_id and parent_id != NULL_REVISION:
+                        parent = self.branch.repository.get_revision(parent_id)
+                        try:
+                            str = ' (' + parent.properties['branch-nick'] + ')'
+                        except KeyError:
+                            str = ""
 
-                    item = gtk.MenuItem(parent.message.split("\n")[0] + str)
-                    item.connect('activate', self._set_revision_cb, parent_id)
-                    prev_menu.add(item)
+                        item = gtk.MenuItem(parent.message.split("\n")[0] + str)
+                        item.connect('activate', self._set_revision_cb, parent_id)
+                        prev_menu.add(item)
                 prev_menu.show_all()
             else:
                 self.prev_rev_action.set_sensitive(False)
@@ -349,12 +348,35 @@ class BranchWindow(Window):
 
             self.next_button.set_menu(next_menu)
 
-            tags = []
-            if self.branch.supports_tags():
-                tagdict = self.branch.tags.get_reverse_tag_dict()
-                if tagdict.has_key(revision.revision_id):
-                    tags = tagdict[revision.revision_id]
-            self.revisionview.set_revision(revision, tags, children)
+            self.revisionview.set_revision(revision)
+            self.revisionview.set_children(children)
+    
+    def _tree_revision_activated(self, widget, path, col):
+        # TODO: more than one parent
+        """Callback for when a treeview row gets activated."""
+        revision = self.treeview.get_revision()
+        parents  = self.treeview.get_parents()
+
+        if len(parents) == 0:
+            parent_id = None
+        else:
+            parent_id = parents[0]
+
+        self.show_diff(revision.revision_id, parent_id)
+        self.treeview.grab_focus()
+        
+    def _menu_diff_cb(self,w):
+        (path, focus) = self.treeview.treeview.get_cursor()
+        revid = self.treeview.model[path][treemodel.REVID]
+        
+        parentids = self.branch.repository.revision_parents(revid)
+
+        if len(parentids) == 0:
+            parentid = NULL_REVISION
+        else:
+            parentid = parentids[0]
+        
+        self.show_diff(revid,parentid)    
 
     def _back_clicked_cb(self, *args):
         """Callback for when the back button is clicked."""
@@ -364,15 +386,32 @@ class BranchWindow(Window):
         """Callback for when the forward button is clicked."""
         self.treeview.forward()
 
-    def _go_clicked_cb(self, revid):
+    def _go_clicked_cb(self, w, p):
         """Callback for when the go button for a parent is clicked."""
-        self.treeview.set_revision_id(revid)
+        if self.revisionview.get_revision() is not None:
+            self.treeview.set_revision(self.revisionview.get_revision())
 
     def _show_clicked_cb(self, revid, parentid):
         """Callback for when the show button for a parent is clicked."""
-        self.treeview.show_diff(revid, parentid)
+        self.show_diff(revid, parentid)
         self.treeview.grab_focus()
 
+    def _compare_with_cb(self,w):
+        """Callback for revision 'compare with' menu. Will show a small
+            dialog with branch revisions to compare with selected revision in TreeView"""
+        
+        from bzrlib.plugins.gtk.revbrowser import RevisionBrowser
+        
+        rb = RevisionBrowser(self.branch,self)
+        ret = rb.run()
+        
+        if ret == gtk.RESPONSE_OK:          
+            (path, focus) = self.treeview.treeview.get_cursor()
+            revid = self.treeview.model[path][treemodel.REVID]
+            self.show_diff(revid, rb.selected_revid)
+            
+        rb.destroy()
+            
     def _set_revision_cb(self, w, revision_id):
         self.treeview.set_revision_id(revision_id)
 
@@ -391,26 +430,24 @@ class BranchWindow(Window):
     def _tag_revision_cb(self, w):
         try:
             self.treeview.set_sensitive(False)
-            self.branch.unlock()
             dialog = AddTagDialog(self.branch.repository, self.treeview.get_revision().revision_id, self.branch)
             response = dialog.run()
             if response != gtk.RESPONSE_NONE:
                 dialog.hide()
             
                 if response == gtk.RESPONSE_OK:
-                    try:
-                        self.branch.lock_write()
-                        self.branch.tags.set_tag(dialog.tagname, dialog._revid)
-                    finally:
-                        self.branch.unlock()
+                    self.treeview.add_tag(dialog.tagname, dialog._revid)
                 
                 dialog.destroy()
 
         finally:
-            self.branch.lock_read()
-            self.treeview.emit("revision-selected")
             self.treeview.set_sensitive(True)
 
+    def _about_dialog_cb(self, w):
+        from bzrlib.plugins.gtk.about import AboutDialog
+
+        AboutDialog().run()
+
     def _col_visibility_changed(self, col, property):
         self.config.set_user_option(property + '-column-visible', col.get_active())
         self.treeview.set_property(property + '-column-visible', col.get_active())
@@ -427,4 +464,43 @@ class BranchWindow(Window):
         dialog.run()
 
     def _refresh_clicked(self, w):
-        self.treeview.update()
+        self.treeview.refresh()
+
+    def _update_tags(self):
+        menu = gtk.Menu()
+
+        if self.branch.supports_tags():
+            tags = self.branch.tags.get_tag_dict().items()
+            tags.sort()
+            tags.reverse()
+            for tag, revid in tags:
+                tag_image = gtk.Image()
+                tag_image.set_from_file(icon_path('tag-16.png'))
+                tag_item = gtk.ImageMenuItem(tag.replace('_', '__'))
+                tag_item.set_image(tag_image)
+                tag_item.connect('activate', self._tag_selected_cb, revid)
+                menu.add(tag_item)
+            self.go_menu_tags.set_submenu(menu)
+
+            self.go_menu_tags.set_sensitive(len(tags) != 0)
+        else:
+            self.go_menu_tags.set_sensitive(False)
+
+        self.go_menu_tags.show_all()
+
+    def show_diff(self, revid=None, parentid=None):
+        """Open a new window to show a diff between the given revisions."""
+        from bzrlib.plugins.gtk.diff import DiffWindow
+        window = DiffWindow(parent=self)
+
+        if parentid is None:
+            parentid = NULL_REVISION
+
+        rev_tree    = self.branch.repository.revision_tree(revid)
+        parent_tree = self.branch.repository.revision_tree(parentid)
+
+        description = revid + " - " + self.branch.nick
+        window.set_diff(description, rev_tree, parent_tree)
+        window.show()
+
+
